@@ -1,5 +1,5 @@
-# GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006  Tom Cato Amundsen
+# GNU Solfege - free ear training software
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,12 +24,10 @@ the mpd module:
 	MusicalPitch
 	MusicDisplayer
 """
+
 if __name__ == '__main__':
-    import sys
-    sys.path.append(".")
-    sys.path.append("src")
-    import i18n
-    i18n.setup(".")
+    import testsetup
+
 
 LOWEST_NOTENAME="c,,,,"
 HIGHEST_NOTENAME="g''''''"
@@ -40,7 +38,8 @@ import soundcard
 from musicalpitch import MusicalPitch, InvalidNotenameException
 from interval import Interval
 from mpdutils import notename_to_int, int_to_notename, int_to_user_notename
-
+from rat import Rat
+from track import Track
 from _exceptions import MpdException
 
 def music_to_tracklist(music, velocity):
@@ -114,9 +113,6 @@ def _test():
     import doctest, musicalpitch, duration, interval
     for mod in (duration, musicalpitch, interval, ):
         doctest.testmod(mod)
-        # FIXME calling setup should not be necessary here.
-        # But doctest.testmod does something strange with _.
-        i18n.setup(".")
 
 if __name__ == '__main__':
     _test()
