@@ -31,7 +31,7 @@ False
 if __name__ == "__main__":
     import testsetup
 
-from soundcard.rat import Rat
+from rat import Rat
 
 class Duration:
     def __init__(self, nh, dots, tuple=Rat(1, 1)):
