@@ -1,5 +1,5 @@
-# GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006  Tom Cato Amundsen
+# GNU Solfege - free ear training software
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin ST, Fifth Floor, Boston, MA  02110-1301  USA
 
+from mpd.track import MidiEventStream
 class Synth:
     NUM_CHANNELS = 16
     def __init__(self, verbose_init):
@@ -24,10 +25,10 @@ class Synth:
         pass
     def stop(self):
         pass
-    def play_track(self, eventstream):
-        print "FakeSynth.play_track(%s)" % eventstream
+    def play_track(self, *tracks):
+        print "FakeSynth.play_track(%s)" % tracks
         last = None
-        for e in eventstream:
+        for e in MidiEventStream(*tracks):
             if e[0] != last:
                 print
             print e,
