@@ -1,5 +1,5 @@
-# GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006  Tom Cato Amundsen
+# GNU Solfege - free ear training software
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,14 +19,16 @@ import gtk, gobject
 import gu
 import inputwidgets
 from specialwidgets import RandomTransposeDialog
+from exceptiondialog import ExceptionDialog
 import soundcard, mpd
 import dataparser
 
 import const, utils
 import random
 import os
+import locale
 import sys
-import signal
+import traceback
 import cfg
 import lessonfile
 import osutils
@@ -103,7 +105,7 @@ class Teacher(cfg.ConfigUtils):
             return
         self.m_P = None
         try:
-            self.m_P = self.lessonfileclass(self.m_lessonfile, self)
+            self.m_P = self.lessonfileclass(self.m_lessonfile)
         # We don't have to check for LessonfileExceptions here because
         # the global exception hook will catch them.
         except IOError:
@@ -464,28 +466,35 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
         spin.set_sensitive(b.get_active())
         hbox.pack_start(spin, False)
     def _lessonfile_exception(self, exception, sourcefile, lineno):
-        msg = _('Please check question number %(idx)i in the lesson file "%(lf)s".\n\n') % {'idx': self.m_t.m_P._idx+1, 'lf': self.m_t.m_P.m_filename}
+        if self.m_t.m_P._idx is None:
+            msg = ""
+        else:
+            msg = _('Please check question number %(idx)i in the lesson file "%(lf)s".\n\n') % {'idx': self.m_t.m_P._idx+1, 'lf': self.m_t.m_P.m_filename}
         msg += _('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno}
-        m = gtk.MessageDialog(self.g_win, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
-                              gtk.BUTTONS_CLOSE, str(exception))
-        m.format_secondary_text(msg)
-        m.set_title(sys.exc_info()[0].__name__)
+        m = ExceptionDialog(exception)
+        m.add_text(msg)
+        if 'm_nonwrapped_text' in dir(exception):
+            m.add_nonwrapped_text(exception.m_nonwrapped_text)
         m.run()
         m.destroy()
     def _mpd_exception(self, exception, sourcefile, lineno):
-        msg = _('Failed to parse the music for question number %(idx)i in the lesson file "%(lf)s".\n\n') % {'idx': self.m_t.m_P._idx+1, 'lf': self.m_t.m_P.m_filename}
-        msg += _('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno}
-        m = gtk.MessageDialog(self.g_win, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
-                              gtk.BUTTONS_CLOSE, str(exception))
-        m.format_secondary_text(msg)
-        m.set_title(sys.exc_info()[0].__name__)
+        m = ExceptionDialog(exception)
+        if 'm_mpd_varname' in dir(exception):
+            m.add_text(_('Failed to parse the music in the variable "%(varname)s" in question number %(idx)i in the lesson file "%(lf)s".') % {
+                'idx': self.m_t.m_P._idx+1, 
+                'lf': self.m_t.m_P.m_filename, 
+                'varname': exception.m_mpd_varname})
+        else:
+            m.add_text(_('Failed to parse the music for question number %(idx)i in the lesson file "%(lf)s".') % {'idx': self.m_t.m_P._idx+1, 'lf': self.m_t.m_P.m_filename})
+        if 'm_mpd_badcode' in dir(exception):
+            m.add_nonwrapped_text(exception.m_mpd_badcode)
+        m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno})
         m.run()
         m.destroy()
     def run_exception_handled(self, method, *args, **kwargs):
         """
         Call method() and catch exceptions with standard_exception_handler.
         """
-        import traceback
         try:
             return method(*args, **kwargs)
         except lessonfile.LessonfileException, e:
@@ -497,16 +506,25 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
         except Exception, e:
             t = traceback.extract_stack()
             if not self.standard_exception_handler(e, __file__):
-                raise e
+                raise
     def standard_exception_handler(self, e, sourcefile, cleanup_function=lambda: False):
+        """
+        Use this method to try to catch a few common solfege exceptions.
+        It should only be used to catch exceptions after the file has
+        successfully parsed by the lessonfile parser, and only used in
+        exercises where the mpd code that might be wrong is comming from
+        lesson files, not generated code.
+
+        Usage:
+        try:
+            do something
+        except Exception, e:
+            if not self.standard_exception_handler(e, __file__):
+                raise
+        """
+        sourcefile = sourcefile.decode(locale.getpreferredencoding(), 'replace')
         if self.m_t.m_app.m_options.disable_exception_handler:
-            import traceback
-            print >> sys.stderr, "standard_exception_handler"
-            print >> sys.stderr, "=========================="
-            traceback.print_exc(file=sys.stderr)
-            print >> sys.stderr, "=========================="
-            print >> sys.stderr, "The above exception was raised in %s:%i" % (sourcefile, sys.exc_info()[2].tb_lineno)
-            return True
+            return False
         if isinstance(e, lessonfile.LessonfileException):
             cleanup_function()
             self._lessonfile_exception(e, sourcefile, sys.exc_info()[2].tb_lineno)
@@ -734,7 +752,9 @@ class LessonbasedGui(Gui):
                 self.g_music_displayer.display(self.m_t.m_P.get_music(varname), fontsize)
             else:
                 self.g_win.display_in_musicviewer(self.m_t.m_P.get_music(varname))
-        except Exception, e:
+        except mpd.MpdException, e:
+            e.m_mpd_varname = varname
+            e.m_mpd_badcode = self.m_t.m_P.get_question()[varname].m_musicdata
             if not self.standard_exception_handler(e, __file__):
                 raise e
     def do_at_question_start_show_play(self):
@@ -742,6 +762,8 @@ class LessonbasedGui(Gui):
         This method is shared by idbyname and elembuilder, and possibly
         other exercises later. It will show and/or play music based on
         the header.at_question_start  variable.
+
+        It might raise mpd.MpdExceptions.
         """
         if self.m_t.m_P.header.at_question_start == 'show':
             self.show_answer()
