@@ -1,5 +1,5 @@
-# GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006  Tom Cato Amundsen
+# GNU Solfege - free ear training software
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,9 +17,11 @@
 
 import configureoutput
 import errno, sys
+import locale
 import shutil
 import soundcard, mpd
 import lessonfile
+import dataparser
 import statistics
 import abstract
 import os
@@ -27,7 +29,10 @@ import cfg, utils
 import webbrowser
 import gu
 from osutils import *
+import reportlib
+import const
 
+print "FIXME: review and mark strings in export_training_set for translation"
 def linux_play_wav_file(filename):
     if not cfg.get_string('sound/wav_player'):
         raise ExecutableDoesNotExist('')
@@ -131,10 +136,7 @@ class SolfegeApp(cfg.ConfigUtils):
             soundcard.synth.error_report_cb = self.m_ui.display_error_message
         elif cfg.get_string("sound/type") == '':
             self.m_ui.display_error_message(
-"""You should configure sound
-from the 'Sound' page of
-the preferences window.
-""")
+_("You should configure sound from the 'Sound' page of the preferences window."))
         elif cfg.get_string("sound/type") == "sequencer-device":
             if not configureoutput.HAVE_LINUX_AWE_VOICE_H and \
                     cfg.get_string("sound/card_info") == "awe":
@@ -151,10 +153,11 @@ the preferences window.
     def display_sound_init_error_message(self, e):
         if isinstance(e, soundcard.SoundInitException):
             self.m_ui.display_error_message(
-            """%s""" % e)
+            """%s""" % str(e).decode(locale.getpreferredencoding(), 'replace'))
         elif isinstance(e, ImportError):
+            estr = str(e).decode(locale.getpreferredencoding(), 'replace')
             self.m_ui.display_error_message("%(message)s\n%(exception)s\n\n%(todo)s" % {
-              'exception':e,
+              'exception': estr,
               'message': "<b>%s</b>" % _("Error loading python module:"),
               'todo': _("You should configure sound from the preferences window, and try to use an external midi player. Or try to recompile the program and check for error messages to see why the module is not built."),
               })
@@ -162,13 +165,13 @@ the preferences window.
             self.m_ui.display_error_message(
 """The sound init failed: %s
 The errno EACCES indicates that you don't have write
-permission to the device.""" % e)
+permission to the device.""" % str(e).decode(locale.getpreferredencoding(), 'replace'))
         elif e.errno == errno.EBUSY:
             self.m_ui.display_error_message(
 """The sound init failed: %s
 It seems like some other program is using the device. You
 should try to quit that other program and restart Solfege."""
- % e)
+ % str(e).decode(locale.getpreferredencoding(), 'replace'))
         else:
             self.m_ui.display_error_message(
 """The sound init failed: 
@@ -177,7 +180,7 @@ You should configure sound from the 'Sound' page of the preferences window.
 
 It is also possible that the OS sound setup is incorrect.
 """
- % (e))
+ % str(e).decode(locale.getpreferredencoding(), 'replace'))
     def play_happy_sound(self):
         mpd.play_music(r"\staff\relative c'{c16 e g a}", 180, 8,
                cfg.get_int('config/preferred_instrument_velocity'))
@@ -204,6 +207,7 @@ It is also possible that the OS sound setup is incorrect.
             self.m_ui.box_dict[self.m_running_exercise].on_end_practise()
         if module not in self.m_teachers:
             self.create_teacher(module)
+        if module not in self.m_ui.box_dict:
             self.m_ui.initialise_exercise(self.m_teachers[module])
         self.m_teachers[module].set_lessonfile(
             self.lessonfile_manager.get(lesson_id, 'filename'))
@@ -215,8 +219,9 @@ It is also possible that the OS sound setup is incorrect.
     def practise_lesson_id(self, lesson_id):
         try:
             module = self._practise_lesson_id(lesson_id)
-        except lessonfile.LessonfileException, e:
-            self.m_ui.display_error_message(str(e))
+        except (lessonfile.LessonfileParseException,
+                dataparser.DataparserException), e:
+            self.m_ui.display_exception_message(e, __file__)
             module = self.lessonfile_manager.get(lesson_id, 'module')
         if not self.m_teachers[module].m_P:
             for n in self.m_ui.box_dict[module].action_area.get_children():
@@ -233,7 +238,8 @@ It is also possible that the OS sound setup is incorrect.
         self.m_ui.box_dict[module].on_start_practise()
         self.m_ui.enter_test_mode()
     def handle_href(self, href, display_docfile_set_adj=1):
-        urlobj = utils.Url(href)
+        # All generatet docbook documents are in utf-8
+        urlobj = utils.Url(href.decode("utf-8"))
         if not urlobj.protocol:
             if urlobj.filename.endswith(".midi"):
                 if os.path.exists(os.path.join(self.m_ui.box_dict['docviewer'].m_htmlwidget.m_document_wd, urlobj.filename)):
@@ -265,6 +271,18 @@ It is also possible that the OS sound setup is incorrect.
         else:
             print "unknown link type", urlobj.protocol
     def handle_solfege_href(self, urlobj):
+        """
+        This method handles three kins of urls:
+        solfege:all-lessonfiles
+            will display a page with links to all installed lesson files
+        solfege:practise/lesson_id
+            practise the exercise with the lesson id 'lesson_id'
+        solfege:practise/path/to/lessonfile
+            Practise a specific lessonfile. Relative path:
+                solfege:practise/lesson-files/chord-min-major
+            Absolute path:
+                solfege:practise//home/user/src/solfege/lesson-files/filename
+        """
         self.m_ui.hide_help_on_current()
         if urlobj.action == 'all-lessonfiles':
             self.m_ui.display_html(self.lessonfile_manager.m_htmldoc)
@@ -276,9 +294,9 @@ It is also possible that the OS sound setup is incorrect.
             return
         if urlobj.lessonfile:
             if not os.path.isfile(urlobj.lessonfile):
-                self.m_ui.display_error_message("Lessonfile not found:\n%s" % urlobj.lessonfile)
+                self.m_ui.display_error_message(_("Lessonfile not found:\n%s") % urlobj.lessonfile)
                 return
-        p = lessonfile.LessonfileCommon(None)#FIXME it would have been
+        p = lessonfile.LessonfileCommon(urlobj.lessonfile)#FIXME it would have been
         # better if we didn't have to parse the file twice.
         p.parse_file(urlobj.lessonfile)
         self._practise_lesson_id(p.header.lesson_id, urlobj)
@@ -310,4 +328,113 @@ It is also possible that the OS sound setup is incorrect.
             self.m_ui.box_dict[self.m_ui.m_viewer].on_end_practise()
         if soundcard.synth:
             soundcard.synth.close()
-
+    def export_training_set(self, export_data, export_dir, output_format):
+        """
+        This function requires a program that can create WAV files
+        from MIDI files and MP3 files from WAV.
+        """
+        def delay(n, tempo):
+            """
+            tempo is a dict of two integers
+            """
+            track = mpd.Track()
+            track.set_bpm(*tempo)#self.get_int('config/default_bpm'))
+            track.note(mpd.Rat(n, 4), 80, 0)
+            soundcard.synth.play_track(track)
+        track_idx = 0
+        num = sum([x['count'] for x in export_data])
+        # MainWin will set this to True if the user want to cancel
+        # the export.
+        self.m_abort_export = False
+        report = reportlib.Report()
+        report.append(reportlib.Heading(1, "Exported exercises"))
+        table = reportlib.Table()
+        report.append(table)
+        for lesson_info in export_data:
+            lesson_id = lesson_info['lesson_id']
+            module = self.lessonfile_manager.get(lesson_id, 'module')
+            if module not in self.m_teachers:
+                self.create_teacher(module)
+            p = self.m_teachers[module].lessonfileclass(
+                self.lessonfile_manager.get(lesson_id, 'filename'))
+            for c in range(lesson_info['count']):
+                if module == 'idbyname':
+                    p.select_random_question()
+                    if p.header.lesson_heading:
+                        s = p.header.lesson_heading
+                    else:
+                        s = p.header.title
+                    table.append_row("%i" % track_idx,
+                                     p.get_question()['name'],
+                                     s)
+                    soundcard.synth.start_export(os.path.join(
+                            export_dir, "track-%i.mid" % track_idx))
+                    for n in range(lesson_info.get('repeat', 1)):
+                        p.play_question()
+                        if n != lesson_info.get('repeat', 1) - 1:
+                            if 'delay' in lesson_info:
+                                delay(lesson_info['delay'], p.get_tempo())
+                    soundcard.synth.end_export()
+                elif module in ('melodicinterval', 'harmonicinterval'):
+                    t = self.m_teachers[module]
+                    t.new_question("c", "c''")
+                    t.q_status = const.QSTATUS_SOLVED
+                    try:
+                        table.append_row("%i" % track_idx, "%s" % utils.int_to_intervalname(t.m_interval))
+                    except AttributeError:
+                        table.append_row("%i" % track_idx, "%s" % (" + ".join([utils.int_to_intervalname(q, False, True) for q in t.m_question])))
+                    soundcard.synth.start_export(os.path.join(
+                            export_dir, "track-%i.mid" % track_idx))
+                    for n in range(lesson_info.get('repeat', 1)):
+                        t.play_question()
+                        if n != lesson_info.get('repeat', 1) - 1:
+                            if 'delay' in lesson_info:
+                                delay(lesson_info['delay'],
+                                    (self.get_int('config/default_bpm'), 4))
+                    soundcard.synth.end_export()
+                else:
+                    print "ignoring exercise with module='%s'" % module
+#####
+                def do_convert(from_format, to_format):
+                    """
+                    Return False if we think the convert failed.
+                    """
+                    filter_str = "app/%s_to_%s_cmd" % (from_format, to_format)
+                    if from_format == 'midi':
+                        from_ext = 'mid'
+                    else:
+                        from_ext = from_format
+                    to_ext = to_format
+                    if not cfg.get_string(filter_str):
+                        self.m_ui.display_error_message2("Config variable not defined", "The missing or empty variable was '%s'" % filter_str)
+                        return False
+                    try:
+                        s = cfg.get_string(filter_str) % {
+                            'in': os.path.join(export_dir, "track-%i.%s" % (track_idx, from_ext)),
+                            'out': os.path.join(export_dir, "track-%i.%s" % (track_idx, to_ext))}
+                        os.system(s)
+                        if os.path.exists(os.path.join(export_dir, "track-%i.%s" % (track_idx, to_ext))):
+                            os.remove(os.path.join(export_dir, "track-%i.%s" % (track_idx, from_ext)))
+                        else:
+                            # This means that the program failed to generate
+                            # the WAV file. We set output_format to 'midi'
+                            # because we don't want to display this error for
+                            # every single file.
+                            output_format = 'midi'
+                            self.m_ui.display_error_message2("External program must have failed", "The file in %(from)s format was not generated from the %(to)s file as expected. Please check your setup." % {'to':to_format.upper(), 'from': from_format.upper()})
+                    except (TypeError, KeyError), e:
+                        self.m_ui.display_error_message2("%(from)s to %(to)s config error", "There was a format string error. Will not generate WAV files. Please check the app/midi_to_wav_cmd config variable." % {'from': from_format, 'to': to_format})
+                        output_format = 'midi'
+                    return True
+#####
+                if output_format in ('mp3', 'wav', 'ogg'):
+                    do_convert('midi', 'wav')
+                if output_format in ('mp3', 'ogg'):
+                    if not do_convert('wav', output_format):
+                        output_format = 'wav'
+                track_idx += 1
+                yield 1.0 * track_idx / num
+                if self.m_abort_export:
+                    del self.m_abort_export
+                    return
+        reportlib.HtmlReport(report, os.path.join(export_dir, "toc.html"))
