@@ -1,3 +1,4 @@
+# coding: utf-8
 # GNU Solfege - free ear training software
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2007  Tom Cato Amundsen
 #
@@ -76,8 +77,10 @@ True
 """
 
 import re
+import codecs
 import os, os.path, sys
 import gethomedir
+import unittest
 
 section_re = re.compile("^\[([\w-]*?)\]")
 value_re = re.compile(  "^([\w-]*?)=(.*)")
@@ -87,7 +90,7 @@ def split(key):
     return key.split("/")
 
 def parse_file_into_dict(dictionary, filename):
-    f = file(filename, "r")
+    f = codecs.open(filename, "rU", "utf-8")
     line = f.readline()
     section = None
     while line:
@@ -408,7 +411,24 @@ class ConfigUtils(object):
         except SyntaxError:
             return []
 
+class Test_parse_file_into_dict(unittest.TestCase):
+    def test_fail_on_non_utf8(self):
+        d = {}
+        try:
+            parse_file_into_dict(d, "test-files/test1.cfg")
+        except UnicodeDecodeError, e:
+            self.assertRaises(NameError, lambda: data)
+    def test_parse_utf8(self):
+        d = {}
+        parse_file_into_dict(d, "test-files/test2.cfg")
+        self.assertEquals(d['sound']['i'], u'3')
+        self.assertEquals(d['sound']['f'], u'1.1')
+        self.assertEquals(d['sound']['s'], u'/home/Us\xe9r/bin/prog')
+        self.assert_(isinstance(d['sound']['s'], basestring))
+        self.assert_(isinstance(d['sound']['s'], unicode))
 
 if __name__ == '__main__':
     import doctest, cfg
     print doctest.testmod(cfg)
+    #import pdb; pdb.set_trace()
+    unittest.main()
