@@ -456,7 +456,7 @@ class Gui(abstract.LessonbasedGui):
         except Exception, e:
             if not self.standard_exception_handler(e, __file__,
                     exception_cleanup):
-                raise e
+                raise
     def on_start_practise(self):
         self.m_t.m_custom_mode = self.get_bool('gui/expert_mode')
         for question in self.m_t.m_P.m_questions:
