@@ -154,6 +154,7 @@ class Gui(abstract.LessonbasedGui):
     def new_question(self, widget=None):
         def exception_cleanup():
             soundcard.synth.stop()
+            self.g_new.set_sensitive(True)
             self.g_give_up.set_sensitive(False)
             self.g_repeat.set_sensitive(False)
             self.g_repeat_arpeggio.set_sensitive(False)
@@ -179,14 +180,11 @@ class Gui(abstract.LessonbasedGui):
             elif g == Teacher.ERR_PICKY:
                 self.g_flashbar.flash(_("You have to solve this question first."))
         except lessonfile.LessonfileException, e:
-            exception_cleanup()
-            # This exception will be raised if for example the music
-            # type for a question is invalid.
-            self.g_win.display_exception_message(e, __file__)
+            if not self.standard_exception_handler(e, __file__, exception_cleanup):
+                raise
         except mpd.MpdException, e:
-            exception_cleanup()
-            self.g_win.display_question_music_error_message(
-               self.m_t.m_P._idx, self.get_string('lessonfile'), e)
+            if not self.standard_exception_handler(e, __file__, exception_cleanup):
+                raise
     def update_gui_after_lessonfile_change(self, *v):
         self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
         self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
@@ -254,11 +252,12 @@ class Gui(abstract.LessonbasedGui):
         except lessonfile.LessonfileException, e:
             # This exception will be raised if for example the music
             # type for a question is invalid.
-            self.g_win.display_exception_message(e, __file__)
+            if not self.standard_exception_handler(e, __file__):
+                raise
         except mpd.MpdException, e:
             # Invalid key variable or music code goes here.
-            self.g_win.display_question_music_error_message(
-                self.m_t.m_P._idx, self.get_string('lessonfile'), e)
+            if not self.standard_exception_handler(e, __file__):
+                raise
     def on_chordtype_clicked(self, btn, t):
         if self.m_t.q_status == const.QSTATUS_NO:
             self.g_flashbar.flash(_("Click 'New chord' to begin."))
