@@ -41,13 +41,12 @@ assignmentlist peek: 'NAME' '='
 # P singchord-1 sparer jeg ikke noe p  ha en peek2_type(t1, t2)
 # som tester de to neste token.
 
-if __name__ == "__main__":
-    import testsetup
-
 import sys
+import traceback
 import i18n
 
 import os, os.path
+import unittest
 import re
 
 tokens = ('NAME', 'STRING', 'OPERATOR', 'INTEGER', 'FLOAT', 'CHAR', 'EOF')
@@ -87,6 +86,12 @@ lastindex_to_group = {LI_INTEGER: 4,
                      LI_FLOAT: 3,
                     }
 
+# Used to find elements in the token tuple
+TOKEN_TYPE = 0
+TOKEN_STRING = 1
+TOKEN_IDX = 2
+TOKEN_LINENO = 3
+
 class _TranslatedString:
     def __init__(self, i18name, cname):
         self.i18name = i18name
@@ -134,63 +139,59 @@ class Question(dict):
             return self["C-name"]
         return "[no name]"
     def get_music(self, varname='music'):
-        return self['m_lessonfile'].prepare_music_from_string(self, varname)
+        return self['m_lessonfile'].get_mpd_music_string(self, varname)
     def is_transposable(question):
         return question['music'].m_musictype in ('chord', 'satb', 'voice', 'rvoice')
 
 
 class DataparserException(Exception):
-    def __init__(self, filename, lexer):
-        Exception.__init__(self)
-        self.m_filename = filename
-        self.m_lexer = lexer
-    def get_err_context(self, lineno):
-        ret = ""
-        if lineno > 0:
-            ret += "\n(line %i): %s" % (lineno-1, self.m_lexer.get_line(lineno-1))
-        ret += "\n(line %i): %s" % (lineno, self.m_lexer.get_line(lineno))
-        s = self.m_lexer.get_line(lineno+1)
-        if s:
-            ret += "\n(line %i): %s" % (lineno+1, s)
-        return ret + "\n"
+    def __init__(self, message):
+        Exception.__init__(self, message)
 
 
 class NameLookupException(DataparserException):
-    def __init__(self, name, parser):
-        DataparserException.__init__(self, parser.m_filename,
-                                     parser._lexer)
-        self.m_name = name
-    def __str__(self):
-        lt = self.m_lexer.m_tokens[self.m_lexer.pos]
-        ret = "Unknown name '%s' in line %i of file '%s':" % (self.m_name, lt[3], self.m_filename)
-        ret += self.get_err_context(lt[3])
-        return ret
+    def __init__(self, parser, bad_pos):
+        DataparserException.__init__(self,
+            _("Unknown name \"%(name)s\" in line %(line)i of file \"%(filename)s\":") % {
+                'name': parser._lexer.m_tokens[bad_pos][TOKEN_STRING],
+                'line': parser._lexer.m_tokens[bad_pos][TOKEN_LINENO],
+                'filename': parser.m_filename})
+        # This variable is only used by the module test code.
+        self.m_token = parser._lexer.m_tokens[bad_pos]
+        self.m_nonwrapped_text = parser._lexer.get_err_context(bad_pos)
 
 
 class DataparserSyntaxError(DataparserException):
-    def __init__(self, parser, msg=""):
-        DataparserException.__init__(self, parser.m_filename,
-                                      parser._lexer)
-        self.m_msg = msg
-    def __str__(self):
-        lt = self.m_lexer.m_tokens[self.m_lexer.pos]
-        if lt == ('EOF', None, None, None):
-            return "Unexpected EOF (end of file)"
-        ret = "Syntax error in line %i of file '%s': %s" % (lt[3], self.m_filename, self.m_msg)
-        ret += self.get_err_context(lt[3])
-        return ret
+    def __init__(self, parser, bad_pos, expect):
+        DataparserException.__init__(self, _('Syntax error in file "%(filename)s". %(expected)s') % {'filename': parser.m_filename, 'expected': expect})
+        # This variable is only used by the module test code.
+        self.m_token = parser._lexer.m_tokens[bad_pos]
+        self.m_nonwrapped_text = parser._lexer.get_err_context(bad_pos)
 
+class AssignmentToReservedWordException(DataparserException):
+    def __init__(self, parser, bad_pos, word):
+        DataparserException.__init__(self, _("Assignment to the reserved word \"%(word)s in the file \"%(filename)s\"") % {'filename': parser.m_filename, 'word': word})
+        # This variable is only used by the module test code.
+        self.m_token = parser._lexer.m_tokens[bad_pos]
+        self.m_nonwrapped_text = parser._lexer.get_err_context(bad_pos)
 
 class UnableToTokenizeException(DataparserException):
-    def __init__(self, lexer, lineno, tok):
-        DataparserException.__init__(self, None, lexer)
-        self.m_lineno = lineno
-        self.m_tok = tok
-    def __str__(self):
-        ret = "In line %i of input: does not recognise this string '%s' as a valid token.'" % (self.m_lineno, self.m_tok)
-        if self.m_lexer.m_tokens:
-            ret += self.get_err_context(self.m_lexer.m_tokens[-1][3])
-        return ret
+    def __init__(self, lexer, lineno, token, pos):
+        """
+        lineno is the zero indexed line number where the exception happened.
+        token is the char that we cannot tokenize
+        pos is the position in the string we are tokenizing.
+        """
+        # This line will add a fake token tuple, so that get_err_context
+        # can produce useful output.
+        lexer.m_tokens.append(('FIXME', token, pos, lineno))
+        # This variable is only used by the module test code.
+        self.m_token = lexer.m_tokens[-1]
+        DataparserException.__init__(self,
+            _('Unable to tokenize line %(lineno)i of the file "%(filename)s"') % {
+                'lineno': lineno + 1,
+                'filename': lexer.m_parser.m_filename})
+        self.m_nonwrapped_text = lexer.get_tokenize_err_context()
 
 
 class Lexer:
@@ -204,6 +205,7 @@ class Lexer:
             src = unicode(src, "UTF-8", errors="replace")
         src = src.replace("\r", "\n")
         self.m_src = src
+        self.pos = 0
         pos = 0
         lineno = 0
         self.m_tokens = []
@@ -224,35 +226,62 @@ class Lexer:
                 break
             m = NEW_re.match(src, pos)
             if not m:
-                raise UnableToTokenizeException(self, lineno, src[pos])
+                raise UnableToTokenizeException(self, lineno, src[pos], pos)
             if m.lastindex == LI_COMMENT:
                 pass
             else:
                 self.m_tokens.append((lastindex_to_ID[m.lastindex],
                          m.group(lastindex_to_group[m.lastindex]), pos, lineno))
             pos = m.end()
-        self.m_tokens.append(("EOF", None, None, None))
-        self.m_tokens.append(("EOF", None, None, None))
-        self.m_tokens.append(("EOF", None, None, None))
-        self.m_tokens.append(("EOF", None, None, None))
-        self.pos = 0
+        self.m_tokens.append(("EOF", None, pos, lineno))
+        self.m_tokens.append(("EOF", None, pos, lineno))
+        self.m_tokens.append(("EOF", None, pos, lineno))
+        self.m_tokens.append(("EOF", None, pos, lineno))
+    def _err_context_worker(self, lexer_pos):
+        ret = ""
+        lineno = self.m_tokens[lexer_pos][TOKEN_LINENO]
+        x = self.m_tokens[lexer_pos][TOKEN_IDX]
+        while x > 0 and self.m_src[x-1] != "\n":
+            x -= 1
+        linestart_idx = x
+        erridx_in_line = self.m_tokens[lexer_pos][TOKEN_IDX] - linestart_idx
+        if lineno > 1:
+            ret += "\n(line %i): %s" % (lineno-1, self.get_line(lineno-2))
+        if lineno > 0:
+            ret += "\n(line %i): %s" % (lineno, self.get_line(lineno-1))
+        ret += "\n(line %i): %s" % (lineno + 1, self.get_line(lineno))
+        ret += "\n" + " " * (erridx_in_line + len("(line %i): " % (lineno+1))) + "^"
+        return ret.strip()
+    def get_tokenize_err_context(self):
+        """
+        return a string with the last part of the file that we were able
+        to tokenize. Used by UnableToTokenizeException
+        """
+        return self._err_context_worker(len(self.m_tokens)-1)
+    def get_err_context(self, pos):
+        return self._err_context_worker(pos)
     def peek(self, forward=0):
         return self.m_tokens[self.pos+forward]
     def peek_type(self, forward=0):
-        return self.m_tokens[self.pos+forward][0]
+        return self.m_tokens[self.pos+forward][TOKEN_TYPE]
     def peek_string(self, forward=0):
-        return self.m_tokens[self.pos+forward][1]
+        return self.m_tokens[self.pos+forward][TOKEN_STRING]
     def scan_any(self):
         """scan the next token"""
         self.pos += 1
-        return self.m_tokens[self.pos-1][1]
+        return self.m_tokens[self.pos-1][TOKEN_STRING]
     def scan(self, t=None):
         """t is the type of token we expect"""
-        if self.m_tokens[self.pos][0] == t:
+        if self.m_tokens[self.pos][TOKEN_TYPE] == t:
             self.pos += 1
-            return self.m_tokens[self.pos-1][1]
+            return self.m_tokens[self.pos-1][TOKEN_STRING]
         else:
-            raise DataparserSyntaxError(self.m_parser, "Token '%s' not found, found '%s' of type %s " %(t, self.m_tokens[self.pos][1], self.m_tokens[self.pos][0]))
+            # Tested in TestLexer.test_scan
+            raise DataparserSyntaxError(self.m_parser, self.pos,
+                _("Token \"%(nottoken)s\" not found, found \"%(foundtoken)s\" of type %(type)s.") % {
+                    'nottoken': t,
+                    'foundtoken': self.m_tokens[self.pos][TOKEN_STRING],
+                    'type': self.m_tokens[self.pos][TOKEN_TYPE]})
     def get_line(self, lineno):
         """line 0 is the first line
         Return an empty string if lineno is out of range.
@@ -263,7 +292,7 @@ class Lexer:
             if self.m_src[idx] == '\n':
                 c += 1
             idx += 1
-        x = idx + 1
+        x = idx
         while x < len(self.m_src) and self.m_src[x] != '\n':
             x += 1
         return self.m_src[idx:x]
@@ -280,6 +309,7 @@ class Dataparser:
         # for example self.blocklists['element'] = []
         self.blocklists = {}
         self.context = self.globals
+        self.m_filename = None
         self.m_ignore_lookup_error = False
     def parse_file(self, filename):
         """We always construct a new parser if we want to parse another
@@ -299,6 +329,11 @@ class Dataparser:
     def prog(self):
         """prog: statementlist EOF"""
         self.statementlist()
+        if self._lexer.peek_type() != 'EOF':
+            # This exception will be raised if we for example have
+            # an extra { after a block definition.
+            raise DataparserSyntaxError(self, self._lexer.pos,
+                    'Expected end of file or statement.')
         self._lexer.scan('EOF')
     def statementlist(self):
         """statementlist: (statement+)"""
@@ -318,7 +353,14 @@ class Dataparser:
                 and self._lexer.peek_type(1) == '(':
             self.include()
         else:
-            raise DataparserSyntaxError(self, "Expected token '=' or '{', found  '%s' of type %s" % (self._lexer.peek_string(), self._lexer.peek_type() ))
+            if self._lexer.peek_type(1) == 'EOF':
+                extra = " Found End of File."
+            else:
+                extra = ""
+            # Add a single A to the end of a valid file to raise
+            # this exception.
+            raise DataparserSyntaxError(self, self._lexer.pos + 1,
+              "Expected token '=' or '{'. %s" % extra)
     def include(self):
         self._lexer.scan_any() # scan include
         self._lexer.scan_any() # scan (
@@ -348,9 +390,11 @@ class Dataparser:
             self.assignment()
     def assignment(self):
         """NAME "=" faktor ("," faktor)* """
+        npos = self._lexer.pos
         name = self._lexer.scan_any()#('NAME')
         if name in self.reserved_words:
-            raise DataparserSyntaxError(self, "Assignment to reserved word '%s'" % name)
+            # do "question = 1" to raise this exception.
+            raise AssignmentToReservedWordException(self, npos, name)
         self._lexer.scan_any()#('=')
         faktorlist = self.faktorlist()
         if len(faktorlist) == 1:
@@ -402,6 +446,7 @@ class Dataparser:
         return faktorlist
     def atom(self):
         """atom: INTEGER | FLOAT | STRING | NAME | FUNCTIONCALL"""
+        npos = self._lexer.pos
         peek = self._lexer.peek_type()
         if peek == 'STRING':
             return self._lexer.scan('STRING')
@@ -412,20 +457,27 @@ class Dataparser:
         elif peek == 'NAME':
             if self._lexer.peek_type(1) == '(':
                 return self.functioncall()
-            return self.lookup_name(self._lexer.scan('NAME'))
+            try:
+                return self.lookup_name(self._lexer.scan('NAME'))
+            except KeyError:
+                # Tested in TestDataParser.test_exception_atom
+                raise NameLookupException(self, npos)
         else:
-            raise DataparserSyntaxError(self, "Expected STRING, INTEGER or NAME+'(', found '%s' of type %s" % (self._lexer.peek_type(), self._lexer.peek_type()))
+            #print "FIXME: have no idea how to raise this exception"
+            raise DataparserSyntaxError(self, npos + 1,
+                "Expected STRING, INTEGER or NAME+'('")
     def functioncall(self):
         """functioncall: NAME "(" faktorlist ")" """
+        npos = self._lexer.pos
         name = self._lexer.scan_any()#'NAME')
         self._lexer.scan('(')
         if self._lexer.peek_type() == ')':
             # functioncall()
             self._lexer.scan(')')
             try:
-                return self.functions(name)()
+                return self.functions[name]()
             except KeyError:
-                raise NameLookupException(name, self)
+                raise NameLookupException(self, npos)
         else:
             # functioncall(arglist)
             arglist = self.faktorlist()
@@ -433,7 +485,7 @@ class Dataparser:
             try:
                 return self.functions[name](*arglist)
             except KeyError:
-                raise NameLookupException(name, self)
+                raise NameLookupException(self, npos)
     def block(self):
         """block: NAME "{" assignmentlist "}" """
         name = self._lexer.scan_any()
@@ -443,7 +495,10 @@ class Dataparser:
             self.questions.append(Question())
             self.context = self.questions[-1]
         else:
-            raise DataparserSyntaxError(self,  "Unknown block name '%s'" % name)
+            if name not in self.blocklists:
+                self.blocklists[name] = []
+            self.blocklists[name].append(dict())
+            self.context = self.blocklists[name][-1]
         self._lexer.scan_any() # scan '{'
         # The question block is a little more code because of the shortcut
         # we allow: question { "music string }
@@ -484,6 +539,9 @@ class Dataparser:
         self._lexer.scan("}")
         self.context = self.globals
     def lookup_name(self, name):
+        """
+        Raises KeyError if the name is not found.
+        """
         if name in self.context:
             return self.context[name]
         elif name in self.globals:
@@ -491,7 +549,7 @@ class Dataparser:
         else:
             if self.m_ignore_lookup_error:
                 return "LOOKUP IGNORED"
-            raise NameLookupException(name, self)
+            raise KeyError
 
 
 def test_tokenizer():
@@ -589,11 +647,153 @@ def main():
             #print "header", p.header
             #print "questions", p.questions
 
+class TestLexer(unittest.TestCase):
+    def test_get_line(self):
+        l = Lexer("""#comment1
+#comment2
+#comment3
+
+var = 3
+""", None)
+        self.assertEquals(l.get_line(0), "#comment1")
+        self.assertEquals(l.get_line(1), "#comment2")
+        self.assertEquals(l.get_line(2), "#comment3")
+        self.assertEquals(l.get_line(3), "")
+        self.assertEquals(l.get_line(4), "var = 3")
+    def test_scan(self):
+        p = Dataparser({}, {})
+        p._lexer = Lexer("\"string\" name 1.2 2 (", p)
+        self.assertEquals(p._lexer.scan(STRING), "string")
+        self.assertEquals(p._lexer.scan(NAME), "name")
+        self.assertEquals(p._lexer.scan(FLOAT), "1.2")
+        self.assertEquals(p._lexer.scan(INTEGER), "2")
+        p._lexer = Lexer("1 2 3", p)
+        try:
+            p._lexer.scan(STRING)
+        except DataparserException, e:
+            self.assertEquals(u"(line 1): 1 2 3\n"
+                              u"          ^",
+                              e.m_nonwrapped_text)
+    def test_unable_to_tokenize(self):
+        p = Dataparser({}, {})
+        try:
+            p._lexer = Lexer("question { a = 3} |!", p)
+        except UnableToTokenizeException, e:
+            self.assertEquals("(line 1): question { a = 3} |!\n"
+                              "                            ^",
+                              e.m_nonwrapped_text)
+        try:
+            p._lexer = Lexer("x = 4\n"
+                             "question { a = 3} |!", p)
+        except UnableToTokenizeException, e:
+            self.assertEquals("(line 1): x = 4\n"
+                              "(line 2): question { a = 3} |!\n"
+                              "                            ^",
+                              e.m_nonwrapped_text)
+
+class TestDataParser(unittest.TestCase):
+    def test_for_trainingset(self):
+        p = Dataparser({}, {})
+        p.parse_string("""fileformat_version = 1
+        lesson {
+        lesson_id = "lkalskdj alskdj lkj "
+        count = 3
+        repeat=2
+        delay = 2}""")
+        self.assertEqual(p.globals['fileformat_version'], 1)
+        self.assertEqual(len(p.blocklists), 1)
+        self.assertEqual(len(p.blocklists['lesson']), 1)
+        l = p.blocklists['lesson'][0]
+        self.assertEqual(l['count'], 3)
+    def assertRaisedIn(self, methodname):
+        t = traceback.extract_tb(sys.exc_info()[2])
+        self.assertEquals(t[-1][2], methodname)
+    def test_exception_statement_1(self):
+        p = Dataparser({}, {})
+        try:
+            p.parse_string("b")
+        except DataparserSyntaxError, e:
+            self.assertRaisedIn('statement')
+            self.assertEquals(u"(line 1): b\n"+
+                              u"           ^",
+                              e.m_nonwrapped_text)
+            self.assertEquals(e.m_token, ('EOF', None, 1, 0))
+    def test_exception_statement_2(self):
+        p = Dataparser({}, {})
+        try:
+            p.parse_string("a)")
+        except DataparserSyntaxError, e:
+            self.assertRaisedIn('statement')
+            self.assertEquals(u"(line 1): a)\n"+
+                              u"           ^",
+                              e.m_nonwrapped_text)
+            self.assertEquals(e.m_token, (')', ')', 1, 0))
+    def test_exception_statement_3(self):
+        p = Dataparser({}, {})
+        try:
+            p.parse_string("""#comment
+  XYZ
+""")
+        except DataparserSyntaxError, e:
+            self.assertRaisedIn('statement')
+            self.assertEquals(u"(line 1): #comment\n"+
+                              "(line 2):   XYZ\n"+
+                              "(line 3): \n"+
+                              "          ^",
+                              e.m_nonwrapped_text)
+    def test_exception_statement_4(self):
+        p = Dataparser({}, {})
+        try:
+            p.parse_string("""#comment
+  A)
+""")
+
+        except DataparserSyntaxError, e:
+            self.assertRaisedIn('statement')
+            self.assertEquals(u"(line 1): #comment\n"+
+                              "(line 2):   A)\n"+
+                              "             ^",
+                              e.m_nonwrapped_text)
+    def test_exception_functioncall(self):
+        p = Dataparser({}, {})
+        try:
+            p.parse_string("a=1\n"
+                           "b = func() c=3")
+        except NameLookupException, e:
+            self.assertEquals(e.m_token[TOKEN_STRING], 'func')
+            self.assertEquals("(line 1): a=1\n"
+                              "(line 2): b = func() c=3\n"
+                              "              ^",
+                              e.m_nonwrapped_text)
+        p = Dataparser({}, {})
+        try:
+            p.parse_string("a=1\n"
+                           "b = func(3, 4) c=3")
+        except NameLookupException, e:
+            self.assertEquals(e.m_token[TOKEN_STRING], 'func')
+            self.assertEquals("(line 1): a=1\n"
+                              "(line 2): b = func(3, 4) c=3\n"
+                              "              ^",
+                              e.m_nonwrapped_text)
+    def test_exception_atom(self):
+        p = Dataparser({}, {})
+        try:
+            p.parse_string("a=b")
+        except NameLookupException, e:
+            self.assertEquals(e.m_token[TOKEN_STRING], "b")
+            self.assertEquals("(line 1): a=b\n"
+                              "            ^",
+                              e.m_nonwrapped_text)
+    def test_exception_assignment(self):
+        p = Dataparser({}, {})
+        try:
+            p.parse_string("question = 3")
+        except AssignmentToReservedWordException, e:
+            self.assertRaisedIn('assignment')
+            self.assertEquals(u"(line 1): question = 3\n" +
+                              u"          ^",
+                              e.m_nonwrapped_text)
+
 if __name__ == '__main__':
-    main()
-    sys.exit()
-    import profile, pstats
-    profile.run("main()", "profile.txt")
-    s = pstats.Stats("profile.txt")
-    s.strip_dirs().sort_stats('cumulative').print_stats(100)
-    #print s.print_callers(.5, "peek_type")
+    import testsetup
+    unittest.main()
