@@ -1,5 +1,5 @@
-# GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006  Tom Cato Amundsen
+# GNU Solfege - free ear training software
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@ import gtk
 import gu
 import mpd, mpd.musicdisplayer
 import abstract, const, lessonfile
-from soundcard.rat import Rat
+from mpd.rat import Rat
 import soundcard
 import utils
 import configureoutput
@@ -83,7 +83,7 @@ class Gui(abstract.LessonbasedGui):
         except Exception, e:
             if not self.standard_exception_handler(e, __file__,
                     self.exception_cleanup):
-                raise e
+                raise
         else:
             self.m_t.q_status = const.QSTATUS_NEW
             self.g_play.set_sensitive(True)
@@ -96,7 +96,7 @@ class Gui(abstract.LessonbasedGui):
         except Exception, e:
             if not self.standard_exception_handler(e, __file__,
                     self.exception_cleanup):
-                raise e
+                raise
         else:
             self.m_t.q_status = const.QSTATUS_NEW
             self.g_play.set_sensitive(True)
@@ -113,7 +113,7 @@ class Gui(abstract.LessonbasedGui):
         except Exception, e:
             if not self.standard_exception_handler(e, __file__,
                     soundcard.synth.stop):
-                raise e
+                raise
     def on_end_practise(self):
         self.m_t.end_practise()
     def show_answer(self, widget=None):
@@ -127,7 +127,7 @@ class Gui(abstract.LessonbasedGui):
                             self.get_int('config/feta_font_size=20'))
         except Exception, e:
             if not self.standard_exception_handler(e, __file__):
-                raise e
+                raise
     def _update(self):
         """
         Updates the buttons above the action_area where you have
@@ -146,7 +146,7 @@ class Gui(abstract.LessonbasedGui):
             except Exception, e:
                 if not self.standard_exception_handler(e, __file__,
                         soundcard.synth.stop):
-                    raise e
+                    raise
         for i in self.g_partbox.get_children():
             i.destroy()
         # if the lessonfile was invalid, m_P could be None
@@ -210,7 +210,7 @@ class Gui(abstract.LessonbasedGui):
                 self.g_music_displayer.clear()
             if not self.standard_exception_handler(e, __file__,
                     self.cleanup_function):
-                raise e
+                raise
         else:
             self.g_play.set_sensitive(True)
             self.g_show.set_sensitive(True)
