@@ -594,7 +594,7 @@ class NewHTMLParser(HTMLParser):
             return
         if data:
             if not isinstance(data, unicode):
-                data = data.decode('utf-8', 'repl]ace')
+                data = data.decode('utf-8', 'replace')
             if self.m_pointer.get().m_name == 'body':
                 p = Block('p', [])
                 p.add(Text(data.strip(), self.in_pre))
