@@ -1,5 +1,5 @@
-# GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 Tom Cato Amundsen
+# GNU Solfege - free ear training software
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 if __name__ == '__main__':
     import testsetup
 
+import unittest
 import glob
 import xrandom
 import uuid
@@ -29,6 +30,9 @@ import gethomedir
 class LessonfileException(Exception):
     pass
 
+class LessonfileParseException(Exception):
+    pass
+
 class NoQuestionInFileException(LessonfileException):
     """
     Raised by find_random_question if the lesson file contains
@@ -53,18 +57,33 @@ class NoQuestionConfiguredException(LessonfileException):
     def __str__(self):
         return "All questions are unselected. Select some questions from the config page."
 
+class UnknownMusicType(LessonfileParseException):
+    """
+    This exception is raised if we encounter an unknown music type.
+    This might happen if the user use the music constructor instead
+    of the predefined functions and spells the music type wrongly.
+    An example:
+
+      music = music("c e g", "cord")
+    """
+    def __init__(self, musictype):
+        LessonfileParseException.__init__(self)
+        self.m_musictype = musictype
+    def __unicode__(self):
+        return u"Unknown music type \"%(musictype)s\"" % {'musictype': self.m_musictype}
+
 import re, os, sys
 import const
 import random
 import mpd
 import cfg
 import utils
-from soundcard.rat import Rat
+from mpd.rat import Rat
 import soundcard
 import dataparser
 import osutils
 
-class plstring(str):
+class plstring(unicode):
     pass
 
 # The keys in the dict say how many steps up or down in the
@@ -161,7 +180,8 @@ lessonfile_functions = {
         '_i': _i, #FIXME
         # play_wav should probably be removed. Replaced by wavfile
         'play_wav': lambda f: Music(cfg.get_string("sound/wav_player") + " " + f, "cmdline"),
-        # Update Music.is_displayable when adding music types
+        # Update Music.is_displayable and is_mpd_parsable
+        # when adding music types
         'music': lambda m, t='normal': Music(m, t),
         'chord': lambda m: Music(m, 'chord'),
         'satb': lambda m: Music(m, 'satb'),
@@ -213,12 +233,28 @@ class _Header(dict):
         #raise NameError(name)
 
 class Music:
+    types = ('normal', 'chord', 'satb', 'voice', 'rvoice', 'rhythm',
+             'percussion', 'cmdline', 'wavfile', 'midifile')
     def __init__(self, musicdata, musictype='normal'):
+        if musictype not in self.types:
+            raise UnknownMusicType(musictype)
         self.m_musicdata = musicdata
         self.m_musictype = musictype
     def is_displayable(self):
+        """
+        Return True if it is possible to display this music object  in a
+        MusicDisplayer. If it returns False, it is possible that this
+        particular music type in theory can be displayed on a
+        MusicDisplayer, but that the code for it has not been written yet.
+        """
         return self.m_musictype in ('normal', 'chord',
                 'satb', 'voice', 'rvoice')
+    def is_mpd_parsable(self):
+        """
+        Return True if the mpd module can parse and play this music object.
+        """
+        return self.m_musictype in ('normal', 'chord',
+                'satb', 'voice', 'rvoice', 'rhythm', 'percussion')
 
 def parse_test_def(s):
     m = re.match("(\d+)\s*x", s)
@@ -226,9 +262,11 @@ def parse_test_def(s):
     return (count, 'x')
 
 class LessonfileCommon(object):
-    def __init__(self, teacher):
+    def __init__(self, filename):
         self.m_prev_question = None
-        self.m_t = teacher
+        # This variable stores the directory the lesson file is located in.
+        # We need this to we can find other files relative to this file.
+        self.m_location = os.path.split(filename)[0]
         self._idx = None
     def parse_file(self, filename):
         """Parse the file named filename. Set these variables:
@@ -237,7 +275,21 @@ class LessonfileCommon(object):
         """
         self.dataparser = dataparser.Dataparser(predef, lessonfile_functions, ('tempo',))
         self.m_filename = filename
-        self.dataparser.parse_file(filename)
+        self._parse_worker(self.dataparser.parse_file, filename)
+    def parse_string(self, s):
+        """
+        See parse_file docstring.
+        """
+        self.dataparser = dataparser.Dataparser(predef, lessonfile_functions, ('tempo',))
+        self.m_filename = "<STRING>"
+        self._parse_worker(self.dataparser.parse_string, s)
+    def _parse_worker(self, func, data):
+        try:
+            func(data)
+        except LessonfileParseException, e:
+            e.m_nonwrapped_text = self.dataparser._lexer.get_err_context(self.dataparser._lexer.pos - 2)
+            e.m_token = self.dataparser._lexer.m_tokens[self.dataparser._lexer.pos - 2]
+            raise
         self.m_transpose = mpd.MusicalPitch.new_from_notename("c'")
         self.header = _Header(self.dataparser.header)
         self.m_globals = self.dataparser.globals
@@ -264,8 +316,8 @@ class LessonfileCommon(object):
             self.header.have_music_displayer = True
 
 class QuestionsLessonfile(LessonfileCommon):
-    def __init__(self, teacher):
-        LessonfileCommon.__init__(self, teacher)
+    def __init__(self, filename):
+        LessonfileCommon.__init__(self, filename)
     def select_random_question(self):
         """
         Select a new question by random. It will use the music in the
@@ -419,20 +471,27 @@ class QuestionsLessonfile(LessonfileCommon):
         """
         assert self._idx is not None
         return self.m_questions[self._idx].get_cname()
-    def prepare_music_from_string(self, question, varname):
+    def get_mpd_music_string(self, question, varname):
+        """
+        Use the musictype and the musicdata of the music objects in the
+        variable named by varname to create a complete music code string
+        that the mpd module can parse.
+        """
+        assert question[varname].is_mpd_parsable()
         if question[varname].m_musictype == 'chord':
             return r"\staff\transpose %s{< %s > }" \
               % (self.m_transpose.get_octave_notename(), question[varname].m_musicdata)
         elif question[varname].m_musictype == 'normal':
-            return self.prepare_music_string_normal_format(question, varname)
+            return self._get_mpd_music_string_normal_format(question, varname)
         elif question[varname].m_musictype == 'satb':
-            return self.prepare_music_string_satb_format(question, varname)
+            return self._get_mpd_music_string_satb_format(question, varname)
         elif question[varname].m_musictype in ('rhythm', 'percussion'):
             return r"\staff{ %s }" % question[varname].m_musicdata
         elif question[varname].m_musictype == 'voice':
             return r"\staff\transpose %s{ %s }" \
               % (self.m_transpose.get_octave_notename(), question[varname].m_musicdata)
-        elif question[varname].m_musictype == 'rvoice':
+        else:
+            assert question[varname].m_musictype == 'rvoice'
             def find_first_note(v):
                 """arg v is a list of the musicdata splittet by whitespace.
                 Return the index of the first note.
@@ -467,23 +526,26 @@ class QuestionsLessonfile(LessonfileCommon):
             return r"\staff\transpose %s\relative %s{ %s }" \
               % (self.m_transpose.get_octave_notename(), 
                  rel_note, " ".join(v))
-        elif question[varname].m_musictype == 'cmdline':
-            return str(question[varname])
-        elif question[varname].m_musictype in ('wavfile', 'midifile'):
-            return question[varname].m_musicdata
-        else:
-            raise LessonfileException("\nUknown music type: '%s'" % question[varname].m_musictype)
     def get_music(self, varname='music'):
         """
         Return the music for the currently selected question. This is complete
         music code that can be fed to mpd.play_music(...).
+
+        If the music type not of a type that mpd.play_music can handle,
+        for example a midi file or a cmdline type, then we return a string
+        that can be used to compare if the music of two questions are equal.
+        This string is not parsable by any functions and should only be used
+        to compare questions.
         """
         assert self._idx is not None
-        return self.prepare_music_from_string(self.m_questions[self._idx], varname)
-    def prepare_music_string_satb_format(self, question, varname):
+        if self.m_questions[self._idx][varname].is_mpd_parsable():
+            return self.get_mpd_music_string(self.m_questions[self._idx], varname)
+        return "%s:%s" % (self.m_questions[self._idx][varname].m_musictype,
+                          self.m_questions[self._idx][varname].m_musicdata)
+    def _get_mpd_music_string_satb_format(self, question, varname):
         """
         It should not be necessary to call this method directly. Call
-        .get_music() or .prepare_music_from_string() and let that
+        .get_music() or .get_mpd_music_string() and let that
         function decide.
         """
         assert question[varname].m_musictype == 'satb'
@@ -504,10 +566,10 @@ class QuestionsLessonfile(LessonfileCommon):
             music = music.replace(r"\addvoice",
                       r"\addvoice\transpose %s" % self.m_transpose.get_octave_notename())
         return music
-    def prepare_music_string_normal_format(self, question, varname):
+    def _get_mpd_music_string_normal_format(self, question, varname):
         """
         It should not be necessary to call this method directly. Call
-        .get_music() or .prepare_music_from_string() and let that
+        .get_music() or .get_mpd_music_string() and let that
         function decide.
         """
         assert question[varname].m_musictype == 'normal'
@@ -564,31 +626,35 @@ class QuestionsLessonfile(LessonfileCommon):
         """
         if not question:
             question = self.get_question()
-        if question[varname].m_musictype == 'cmdline':
-            osutils.run_external_program(str(question[varname].m_musicdata), os.path.split(self.m_t.m_lessonfile)[0], "")
-        elif question[varname].m_musictype == 'wavfile':
-            if os.path.exists(os.path.join(
-                        os.path.split(self.m_t.m_lessonfile)[0],
-                        question[varname].m_musicdata)):
-                    soundcard.play_wav_file(os.path.join(
-                        os.path.split(self.m_t.m_lessonfile)[0],
-                        question[varname].m_musicdata))
+        try:
+            if question[varname].m_musictype == 'cmdline':
+                osutils.run_external_program(str(question[varname].m_musicdata), self.m_location, "")
+            elif question[varname].m_musictype == 'wavfile':
+                if os.path.exists(os.path.join(
+                            self.m_location,
+                            question[varname].m_musicdata)):
+                        soundcard.play_wav_file(os.path.join(
+                            self.m_location,
+                            question[varname].m_musicdata))
+                else:
+                    raise FileNotFound(question[varname].m_musicdata)
+            elif question[varname].m_musictype == 'midifile':
+                soundcard.play_midi_file(os.path.join(
+                    self.m_location,
+                    question[varname].m_musicdata))
+            elif question[varname].m_musictype in 'rhythm':
+                self.play_question_musicformat_rhythm(question, varname)
+            elif question[varname].m_musictype in 'percussion':
+                self.play_question_musicformat_percussion(question, varname)
+            elif question[varname].m_musictype in ('normal', 'satb', 'voice', 'rvoice'):
+                self.play_question_musicformat_normal_satb(question, varname)
             else:
-                raise FileNotFound(question[varname].m_musicdata)
-        elif question[varname].m_musictype == 'midifile':
-            soundcard.play_midi_file(os.path.join(
-                os.path.split(self.m_t.m_lessonfile)[0],
-                question[varname].m_musicdata))
-        elif question[varname].m_musictype in 'rhythm':
-            self.play_question_musicformat_rhythm(question, varname)
-        elif question[varname].m_musictype in 'percussion':
-            self.play_question_musicformat_percussion(question, varname)
-        elif question[varname].m_musictype in ('normal', 'satb', 'voice', 'rvoice'):
-            self.play_question_musicformat_normal_satb(question, varname)
-        elif question[varname].m_musictype in 'chord':
-            self.play_question_musicformat_chord(question, varname)
-        else:
-            raise LessonfileException("Unrecognised musictype '%s'" % question[varname].m_musictype)
+                assert question[varname].m_musictype in 'chord'
+                self.play_question_musicformat_chord(question, varname)
+        except mpd.MpdException, e:
+            e.m_mpd_varname = varname
+            e.m_mpd_badcode = question[varname].m_musicdata
+            raise
     def play_question_musicformat_rhythm(self, question, varname):
         """
         This will play music objects of the type 'rhythm'. See the
@@ -731,13 +797,13 @@ class QuestionsLessonfile(LessonfileCommon):
         Convert instrument names to integer values.
         Returns: lowest, middle, highest
         """
-        if self.m_t.get_bool('config/override_default_instrument'):
-            instrument = [self.m_t.get_int('config/lowest_instrument'),
-                         self.m_t.get_int('config/lowest_instrument_velocity'),
-                         self.m_t.get_int('config/middle_instrument'),
-                         self.m_t.get_int('config/middle_instrument_velocity'),
-                         self.m_t.get_int('config/highest_instrument'),
-                         self.m_t.get_int('config/highest_instrument_velocity')]
+        if cfg.get_bool('config/override_default_instrument'):
+            instrument = [cfg.get_int('config/lowest_instrument'),
+                         cfg.get_int('config/lowest_instrument_velocity'),
+                         cfg.get_int('config/middle_instrument'),
+                         cfg.get_int('config/middle_instrument_velocity'),
+                         cfg.get_int('config/highest_instrument'),
+                         cfg.get_int('config/highest_instrument_velocity')]
         elif 'instrument' in question:
             instrument = question['instrument']
         elif 'instrument' in self.m_globals:
@@ -813,13 +879,13 @@ class HeaderLessonfile(LessonfileCommon):
     This lesson file class should be used by all the exercise modules
     that does not need any question blocks defined.
     """
-    def __init__(self, filename, teacher):
-        LessonfileCommon.__init__(self, teacher)
+    def __init__(self, filename):
+        LessonfileCommon.__init__(self, filename)
         self.parse_file(filename)
 
 class DictationLessonfile(QuestionsLessonfile):
-    def __init__(self, filename, teacher):
-        QuestionsLessonfile.__init__(self, teacher)
+    def __init__(self, filename):
+        QuestionsLessonfile.__init__(self, filename)
         self.parse_file(filename)
     def get_breakpoints(self):
         assert self._idx is not None
@@ -860,13 +926,13 @@ class DictationLessonfile(QuestionsLessonfile):
         self._idx = 0
 
 class SingChordLessonfile(QuestionsLessonfile):
-    def __init__(self, filename, teacher):
-        QuestionsLessonfile.__init__(self, teacher)
+    def __init__(self, filename):
+        QuestionsLessonfile.__init__(self, filename)
         self.parse_file(filename)
 
 class NameIntervalLessonfile(HeaderLessonfile):
-    def __init__(self, filename, teacher):
-        HeaderLessonfile.__init__(self, filename, teacher)
+    def __init__(self, filename):
+        HeaderLessonfile.__init__(self, filename)
         iquality = []
         inumbers = []
         self.header.intervals = [mpd.Interval(n) for n in self.header.intervals]
@@ -901,11 +967,8 @@ class NameIntervalLessonfile(HeaderLessonfile):
             self.header.tones = [mpd.MusicalPitch.new_from_notename(n) for n in self.header.tones]
 
 class IdByNameLessonfile(QuestionsLessonfile, TestSupport):
-    def __init__(self, filename, teacher):
-        """
-        teacher -- the teacher class contructing this lesson file class
-        """
-        QuestionsLessonfile.__init__(self, teacher)
+    def __init__(self, filename):
+        QuestionsLessonfile.__init__(self, filename)
         TestSupport.__init__(self)
         self.parse_file(filename)
     def parse_file(self, filename):
@@ -924,11 +987,11 @@ class IdByNameLessonfile(QuestionsLessonfile, TestSupport):
                 self.m_questions.append(question)
 
 class SingAnswerLessonfile(QuestionsLessonfile):
-    def __init__(self, filename, teacher):
+    def __init__(self, filename):
         """
         teacher -- the teacher class contructing this lesson file class
         """
-        QuestionsLessonfile.__init__(self, teacher)
+        QuestionsLessonfile.__init__(self, filename)
         self.parse_file(filename)
     def parse_file(self, filename):
         super(SingAnswerLessonfile, self).parse_file(filename)
@@ -955,8 +1018,8 @@ class IntervalsLessonfile(HeaderLessonfile, TestSupport):
         self.m_test_idx += 1
 
 class ChordLessonfile(QuestionsLessonfile):
-    def __init__(self, filename, teacher):
-        QuestionsLessonfile.__init__(self, teacher)
+    def __init__(self, filename):
+        QuestionsLessonfile.__init__(self, filename)
         self.parse_file(filename)
     def parse_file(self, filename):
         """
@@ -1001,8 +1064,8 @@ class ChordLessonfile(QuestionsLessonfile):
         return -1
 
 class ElembuilderLessonfile(QuestionsLessonfile):
-    def __init__(self, filename, teacher):
-        QuestionsLessonfile.__init__(self, teacher)
+    def __init__(self, filename):
+        QuestionsLessonfile.__init__(self, filename)
         self.parse_file(filename)
     def parse_file(self, filename):
         super(ElembuilderLessonfile, self).parse_file(filename)
@@ -1024,6 +1087,7 @@ class LessonFileManager:
                 continue
             v = glob.glob(os.path.join(dir, "*"))
             for filename in v:
+                filename = filename.decode(locale.getpreferredencoding())
                 # since I usually run solfege from the source dir:
                 if os.path.split(filename)[-1] in ('.arch-ids', 'Makefile', 'Makefile.in') \
                   or not os.path.isfile(filename):
@@ -1221,20 +1285,20 @@ class LessonIdParser(object):
                 ofile.write(self.m_file_content[m.end():])
             ofile.close()
 
+class TestParser(unittest.TestCase):
+    def test_bad_musictype(self):
+        s = 'question {\n' \
+            'music = music("c e g", "chor")\n' \
+            '}'
+        p = LessonfileCommon("<STRING>")
+        try:
+            p.parse_string(s)
+        except UnknownMusicType, e:
+            self.assertEquals(e.m_token[dataparser.TOKEN_STRING], "chor")
+            self.assertEquals('(line 1): question {\n'
+                              '(line 2): music = music("c e g", "chor")\n'
+                              '                                 ^',
+                              e.m_nonwrapped_text)
+
 if __name__ == '__main__':
-    import time
-    t1 = time.clock()
-    cfg.initialise(None, "default.config",
-        "~/.solfegerc")
-    t2 = time.clock()
-    for n in range(1):
-        p = ElembuilderLessonfile(sys.argv[1], None)
-        #p = LessonFileManager()
-        #p = LessonfileCommon(None)
-        #p.parse_file('lesson-files/chord-min-major')
-        for q in p.m_questions:
-            print q
-        print p.blocklists['element']
-    t3 = time.clock()
-    #print t3-t2
-    #print t2-t1
+    unittest.main()
