@@ -1,7 +1,7 @@
 # -*- coding: iso-8859-1 -*-
 
-# GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006  Tom Cato Amundsen
+# GNU Solfege - free ear training software
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -189,6 +189,7 @@ import rhythmtapping
 import rhythmtapping2
 
 import learning_tree_editor
+from trainingsetdlg import TrainingSetDialog
 from docviewer import DocViewer
 from helpbrowser import HelpBrowser
 import gethomedir
@@ -335,6 +336,8 @@ class MainWin(App, cfg.ConfigUtils):
           ('TreeEditor', None, _('Edit learning tree'), None, None,
             self.do_tree_editor),
           ('TestsMenu', None, _("_Tests")),
+          ('ExportTrainingSet', None, _('New training set editor window'), None, None,
+            self.new_training_set_editor),
           ('OpenPreferencesWindow', 'gtk-preferences', None, '<ctrl>F12', None,
             self.open_preferences_window),
           ('HelpMenu', None, _('_Help')),
@@ -510,17 +513,14 @@ class MainWin(App, cfg.ConfigUtils):
                 })
             import webbrowser
             webbrowser.open_new("http://www.solfege.org?%s" % params)
-    def display_exception_message(self, exception, err_file=None, msg=""):
+    def display_exception_message(self, exception, err_file=None):
         """Call this function only inside an except clause."""
+        from exceptiondialog import ExceptionDialog
+        m = ExceptionDialog(exception)
         if err_file:
-            msg += _('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': err_file, 'lineno': sys.exc_info()[2].tb_lineno}
-        exception_str = unicode(exception)
-        if not isinstance(exception_str, unicode):
-            exception_str = exception_str.decode(locale.getpreferredencoding(), 'replace')
-        m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
-                              gtk.BUTTONS_CLOSE, str(exception))
-        m.set_title(sys.exc_info()[0].__name__)
-        m.format_secondary_text(msg)
+            m.add_text(_('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': err_file.decode(locale.getpreferredencoding(), 'replace'), 'lineno': sys.exc_info()[2].tb_lineno})
+        if 'm_nonwrapped_text' in dir(exception):
+            m.add_nonwrapped_text(exception.m_nonwrapped_text)
         m.run()
         m.destroy()
     def display_error_message2(self, text, secondary_text):
@@ -528,18 +528,34 @@ class MainWin(App, cfg.ConfigUtils):
         This is the new version of display_error_message, and it will
         eventually replace the old.
         """
+        if splash_win and splash_win.props.visible:
+            splash_win.hide()
+            reshow_splash = True
+        else:
+            reshow_splash = False
         if not isinstance(text, unicode):
-            msg = msg.decode(locale.getpreferredencoding(), 'replace')
-        m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
+            text = text.decode(locale.getpreferredencoding(), 'replace')
+        if not isinstance(secondary_text, unicode):
+            secondary_text = secondary_text.decode(locale.getpreferredencoding(), 'replace')
+        m = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                               gtk.BUTTONS_CLOSE, text)
         if secondary_text:
             m.format_secondary_text(secondary_text)
         m.run()
         m.destroy()
+        if reshow_splash:
+            splash_win.show()
+            while gtk.events_pending():
+                gtk.main_iteration(0)
     def display_error_message(self, msg, title=None, secondary_text=None):
+        if splash_win and splash_win.props.visible:
+            splash_win.hide()
+            reshow_splash = True
+        else:
+            reshow_splash = False
         if not isinstance(msg, unicode):
             msg = msg.decode(locale.getpreferredencoding(), 'replace')
-        m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
+        m = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                               gtk.BUTTONS_CLOSE, None)
         m.set_markup(msg)
         if title:
@@ -548,6 +564,10 @@ class MainWin(App, cfg.ConfigUtils):
             m.format_secondary_text(secondary_text)
         m.run()
         m.destroy()
+        if reshow_splash:
+            splash_win.show()
+            while gtk.events_pending():
+                gtk.main_iteration(0)
     def display_question_music_error_message(self, idx, lessonfile, exception, err_file=None):
         """Call this function only inside an except clause."""
         msg = _("Failed to parse the music for question number %(idx)i in '%(lf)s':\n%(ex)s") % {'idx': idx, 'lf': lessonfile, 'ex': exception}
@@ -646,6 +666,9 @@ GNU General Public License for more details.""")
         self.m_viewer = 'docviewer'
         self.box_dict['docviewer'].show()
     def display_docfile(self, fn, anchor):
+        """
+        Display the HTML file named by fn in the help browser window.
+        """
         if not self.g_help_browser:
             self.g_help_browser = HelpBrowser(self.m_app)
         self.g_help_browser.show()
@@ -715,6 +738,10 @@ GNU General Public License for more details.""")
         self.box_dict[self.m_viewer].exit_test_mode()
         # rebuild the menus in case we hav passed a test.
         self.create_practise_and_test_menu()
+    def new_training_set_editor(self, widget):
+        dlg = TrainingSetDialog(self.m_app)
+        dlg.show_all()
+
 
 import locale
 locale.setlocale(locale.LC_NUMERIC, "C")
@@ -741,11 +768,13 @@ if splash_win:
     splash_win.show_progress("creating MainWin")
 
 def start_app(prefix, datadir):
+    global splash_win
     w = MainWin(options, datadir, lessonfile_manager)
     w.show()
     w.post_constructor()
     if splash_win:
         splash_win.destroy()
+        splash_win = None
 
     def ef(t, value, traceback):
         if options.debug:
