@@ -143,7 +143,8 @@ class Gui(abstract.LessonbasedGui):
             else:
                 self.g_win.display_error_message2(_("Could not satisfy the constraints in the lesson header."), 'You must make more tones available by adjusting the "tones" variable in the lesson file header of the lesson file "%s".' % self.m_t.m_P.m_filename)
         except lessonfile.LessonfileException, e:
-            self.g_win.display_exception_message(e, __file__)
+            if not self.standard_exception_handler(e, __file__):
+                raise
     def on_interval_quality_clicked(self, button, n):
         if self.m_t.q_status == const.QSTATUS_NO:
             self.g_flashbar.flash(_("Click 'New' to begin."))
