@@ -10,7 +10,7 @@ class ExecutableDoesNotExist(Exception):
         return _("The executable '%s' does not exist. Please check your setup.") % self.v
 
 
-__all__ = ['run_external_program']
+__all__ = ['run_external_program', 'ExecutableDoesNotExist']
 __child_pid = None
 def run_external_program(cmdstring, wdir, argument):
     global __child_pid
@@ -25,6 +25,9 @@ def run_external_program(cmdstring, wdir, argument):
     if not os.path.exists(os.path.join(wdir, v[0])):
         raise ExecutableDoesNotExist(v[0])
         return
+    if sys.platform == 'win32':
+        os.system(cmdline)
+        return
     try:
         if __child_pid is not None:
             os.kill(__child_pid, signal.SIGKILL)
